@@ -3,6 +3,7 @@
 #include "frontend.h"
 #include "database.h"
 #include "question.h"
+#include "template.h"
 #include "strutl.h"
 #include "debconfclient.h"
 
@@ -54,6 +55,21 @@ static char *_confmodule_process(struct confmodule *mod, char *in)
     return NULL;
 }
 
+/*
+ * @brief mark a file descriptor close-on-exec
+ * @param int fd - file descriptor
+ * @return int - 0 on success, -1 with errno set on error
+ */
+static int _confmodule_cloexec(int fd)
+{
+    int flags = fcntl (fd, F_GETFD);
+    if (flags < 0)
+        return flags;
+    if (fcntl (fd, F_SETFD, flags | FD_CLOEXEC) < 0)
+        return -1;
+    return 0;
+}
+
 /* public functions */
 static int confmodule_communicate(struct confmodule *mod)
 {
@@ -84,7 +100,7 @@ static int confmodule_communicate(struct confmodule *mod)
                 return DC_OK;
             }
 
-            ret = read(mod->infd, buf, sizeof(buf));
+            ret = read(mod->infd, buf, sizeof(buf) - 1);
             if (ret < 0) {
                 if (errno == EINTR)
                     continue;
@@ -180,15 +196,15 @@ static pid_t confmodule_run(struct confmodule *mod, int argc, char **argv)
             DIE("Cannot execute client config script");
             break;
         case 0:
-            /* 20=read/to, 21=write/to, 22=read/from, 23=write/from, 24=null */
+            /* 50=read/to, 51=write/to, 52=read/from, 53=write/from, 54=null */
             config[4] = open("/dev/null", O_RDWR);
             for (i = 0; i < 5; i++)
-                check_fd(config[i], 20 + i, old);
+                check_fd(config[i], 50 + i, old);
             for (i = 0; i <= 2; i++)
-                dup2(old[i] ? i : 24, DEBCONF_OLD_FD_BASE + i);
-            dup2(20, 0); dup2(23, 1); dup2(23, 3);
+                dup2(old[i] ? i : 54, DEBCONF_OLD_FD_BASE + i);
+            dup2(50, 0); dup2(53, 1); dup2(53, 3);
             for (i = 0; i < 5; i++)
-                close(20 + i);
+                close(50 + i);
 
             args = (char **)malloc(sizeof(char *) * argc);
             for (i = 1; i < argc; i++)
@@ -202,6 +218,8 @@ static pid_t confmodule_run(struct confmodule *mod, int argc, char **argv)
             close(config[0]); close(config[3]);
             mod->infd = config[2];
             mod->outfd = config[1];
+            _confmodule_cloexec(mod->infd);
+            _confmodule_cloexec(mod->outfd);
     }
 
     return pid;
@@ -249,7 +267,10 @@ static int confmodule_update_seen_questions(struct confmodule *mod, enum seen_ac
                 DELETE(*(mod->seen_questions + mod->number_seen_questions - 1));
                 (mod->number_seen_questions) --;
                 if (mod->number_seen_questions == 0)
+                {
                     DELETE(mod->seen_questions);
+                    break;
+                }
             }
             break;
         case STACK_SEEN_SAVE:
@@ -281,6 +302,27 @@ static int confmodule_update_seen_questions(struct confmodule *mod, enum seen_ac
 static int confmodule_save(struct confmodule *mod)
 {
     int ret = DC_OK;
+
+    if (!load_all_translations())
+    {
+        /* This isn't entirely accurate; really it should be done in
+         * rfc822db's implementation of mod->templates->methods.save().
+         * However, that doesn't have convenient access to the questions
+         * database, so we do it here instead for the time being.
+         */
+        struct question *q = mod->questions->methods.get(
+            mod->questions, "debconf/translations-dropped");
+        if (q != NULL)
+        {
+            if (strcmp(question_getvalue(q, ""), "true") != 0)
+            {
+                question_setvalue(q, "true");
+                mod->questions->methods.set(mod->questions, q);
+            }
+            question_deref(q);
+        }
+    }
+
     ret |= mod->update_seen_questions(mod, STACK_SEEN_SAVE);
     if (mod->questions)
         ret |= mod->questions->methods.save(mod->questions);
