@@ -34,10 +34,10 @@ struct option g_dpc_args[] = {
 	{ "help", 0, NULL, 'h' },
 	{ "frontend", 1, NULL, 'f' },
 	{ "priority", 1, NULL, 'p' },
-	{ "default-priority", 1, NULL, 'd' },
+	{ "default-priority", 0, NULL, 'd' },
 	{ "all", 0, NULL, 'a' },
 	{ "unseen-only", 0, NULL, 'u' },
-    { "force", 0, NULL, 'F' },
+	{ "force", 0, NULL, 'F' },
 	{ 0, 0, 0, 0 }
 };
 
@@ -58,16 +58,16 @@ static void cleanup(void)
 {
 	if (g_frontend != NULL)
 		frontend_delete(g_frontend);
-    if (g_templates != NULL)
-    {
-        g_templates->methods.save(g_templates);
-        template_db_delete(g_templates);
-    }
-    if (g_questions != NULL)
-    {
-        g_questions->methods.save(g_questions);
-        question_db_delete(g_questions);
-    }
+	if (g_templates != NULL)
+	{
+		g_templates->methods.save(g_templates);
+		template_db_delete(g_templates);
+	}
+	if (g_questions != NULL)
+	{
+		g_questions->methods.save(g_questions);
+		question_db_delete(g_questions);
+	}
 	if (g_config != NULL)
 		config_delete(g_config);
 }
@@ -93,8 +93,8 @@ static void sighandler(int sig)
  ************************************************************************/
 static void usage(void)
 {
-	printf("dpkg-reconfigure [--frontend <frontend>] [--priority <priority>]\n");
-	printf("dpkg-reconfigure [-f <frontend>] [-p <priority>]\n");
+	printf("dpkg-reconfigure [--frontend <frontend>] [--priority <priority>] package\n");
+	printf("dpkg-reconfigure [-f <frontend>] [-p <priority>] package\n");
 	exit(0);
 }
 
@@ -139,7 +139,7 @@ void loadtemplate(const char *filename, const char *owner)
 		{
 			q = question_new(t->tag);
 			q->template = t;
-                        template_ref(t);
+			template_ref(t);
 		}
 		else if (q->template != t)
 		{
@@ -253,12 +253,55 @@ int runconfmodule(int argc, char **argv)
 	confmodule = confmodule_new(g_config, g_templates, g_questions, g_frontend);
 	confmodule->run(confmodule, argc, argv);
 	confmodule->communicate(confmodule);
+	confmodule->shutdown(confmodule);
 	ret = confmodule->exitcode;
 	confmodule_delete(confmodule);
 
 	return ret;
 }
 
+int runscript (const char *pkg, const char *script, const char *param) {
+	int ret;
+	char filename[1024];
+	char *argv[5] = {0};
+	char *version = getfield(pkg, VERSIONFIELD);
+
+	snprintf(filename, sizeof(filename), INFODIR "/%s.%s", pkg, script);
+	if (! file_exists(filename,S_IXUSR|S_IXGRP|S_IXOTH))
+		return 0; /* it's ok if the script doesn't exist */
+	
+	if (strcmp("script", "config") == 0 || is_confmodule(filename))
+	{
+		argv[1] = filename;
+		argv[2] = param;
+		argv[3] = version;
+		if ((ret = runconfmodule(4, argv)) != 0)
+			return ret;
+	}
+	else
+	{
+		/* according to debconf:
+		 * Since a non confmodule might run other programs that
+		 * use debconf, checkpoint the db state and
+		 * reinitialize when the script finishes 
+		 */
+		g_templates->methods.save(g_templates);
+		g_questions->methods.save(g_questions);
+	
+		unsetenv("DEBIAN_HAS_FRONTEND");
+
+		strvacat(filename, sizeof(filename), " ", param, " ", version, NULL);
+		ret = system(filename);
+	
+		setenv("DEBIAN_HAS_FRONTEND", "1", 1);
+	
+		g_templates->methods.load(g_templates);
+		g_questions->methods.load(g_questions);
+	}
+
+	return ret;
+}
+
 /************************************************************************
  * Function: reconfigure
  * Inputs: pkgs - pointer to an array of packages
@@ -270,7 +313,6 @@ int runconfmodule(int argc, char **argv)
 int reconfigure(char **pkgs, int i, int max)
 {
 	int ret;
-	char *argv[5] = {0};
 	char filename[1024];
 	char *pkg;
 
@@ -278,21 +320,6 @@ int reconfigure(char **pkgs, int i, int max)
 	{
 		pkg = pkgs[i++];
 
-		snprintf(filename, sizeof(filename), INFODIR "/%s.config", pkg);
-		if (!file_exists(filename, S_IXUSR|S_IXGRP|S_IXOTH))
-		{
-			INFO(INFO_WARN, "%s is not installed, or does not use debconf", pkg);
-
-                        /* Don't die, though this doesn't have a
-                        config script.. we might be doing stuff with
-                        debian-installer, which doesn't use .config
-                        scripts.  Don't uncomment this without talking
-                        to -boot first
-                        -- tfheen, 2002-09-17 */
-
-                        /* continue; */
-		}
-		/* startup the confmodule; run the config script and talk to it */
 		g_frontend->methods.set_title(g_frontend, pkg);
 		if (strstr(getfield(pkg, STATUSFIELD), " ok installed") == 0)
 			DIE("%s is not fully installed", pkg);
@@ -301,54 +328,18 @@ int reconfigure(char **pkgs, int i, int max)
 		if (file_exists(filename, S_IRUSR|S_IRGRP|S_IROTH))
 			loadtemplate(filename, pkg);
 		
-		snprintf(filename, sizeof(filename), INFODIR "/%s.config", pkg);
-		if (file_exists(filename,S_IXUSR|S_IXGRP|S_IXOTH))
-		{
-			argv[1] = filename;
-			argv[2] = "reconfigure";
-			argv[3] = getfield(pkg, VERSIONFIELD);
-			if ((ret = runconfmodule(4, argv)) != DC_OK)
-				return ret;
-		}
-        else
-        {
-            snprintf(filename, sizeof(filename), INFODIR "/%s.postinst", pkg);
-            if (file_exists(filename,S_IXUSR|S_IXGRP|S_IXOTH))
-            {
-                if (is_confmodule(filename))
-                {
-                    argv[1] = filename;
-                    argv[2] = "configure";
-                    argv[3] = getfield(pkg, VERSIONFIELD);
-                    if ((ret = runconfmodule(4, argv)) != 0)
-                        return ret;
-                }
-                else
-                {
-                    /* according to debconf:
-                     * Since postinst might run other programs that
-                     * use debconf, checkpoint the db state and
-                     * reinitialize when the script finishes 
-                     */
-                    g_templates->methods.save(g_templates);
-                    g_questions->methods.save(g_questions);
-
-                    unsetenv("DEBIAN_HAS_FRONTEND");
-                    setenv("DEBCONF_SHOWOLD", "1", 1);
-
-                    strvacat(filename, sizeof(filename), " configure ", getfield(pkg, VERSIONFIELD), NULL);
-
-                    ret = system(filename);
-                    if (ret != 0) return DC_NOTOK;
-
-                    setenv("DEBIAN_HAS_FRONTEND", "1", 1);
-                    unsetenv("DEBCONF_SHOWOLD");
-
-                    g_templates->methods.load(g_templates);
-                    g_questions->methods.load(g_questions);
-                }
-            }
-        }
+		/* Simulation of reinstalling a package, without bothering with
+		   removing the files and putting them back. Just like in a
+		   regular reinstall, run config, and postinst scripts in
+		   sequence, with args. Do not run postrm, because the postrm
+		   can depend on the package's files actually being gone
+		   already. */
+		if ((ret = runscript(pkg, "prerm", "upgrade")) != 0)
+			return ret;
+		if ((ret = runscript(pkg, "config", "reconfigure")) != 0)
+			return ret;
+		if ((ret = runscript(pkg, "postinst", "configure")) != 0)
+			return ret;
 
 		i++;
 	}
@@ -365,29 +356,44 @@ int reconfigure(char **pkgs, int i, int max)
 int main(int argc, char **argv)
 {
 	int opt, ret;
+	int unseen_only=0;
+	int default_priority=0;
+	char *priority_override=NULL;
 
 	signal(SIGINT, sighandler);
 	setlocale (LC_ALL, "");
 
-    if (getuid() != 0)
-        DIE("%s must be run as root", argv[0]);
+	if (getuid() != 0)
+		DIE("%s must be run as root", argv[0]);
 
 	g_config = config_new();
 
-	while ((opt = getopt_long(argc, argv, "dhf:p:aF", g_dpc_args, NULL)) > 0)
+	while ((opt = getopt_long(argc, argv, "hf:p:duaF", g_dpc_args, NULL)) > 0)
 	{
 		switch (opt)
 		{
 		case 'h': usage(); break;
 		case 'f': setenv("DEBIAN_FRONTEND", optarg, 1); break;
-        case 'p': g_config->set(g_config, "_cmdline::priority", optarg); break;
-		case 'd': break;
-		case 'u': break;
+		case 'p': priority_override=strdup(optarg); break;
+		case 'd': default_priority=1; break;
+		case 'u': unseen_only=1; break;
 		case 'a': opt_all = 1; break;
-        case 'F': opt_force = 1; break;
+		case 'F': opt_force = 1; break;
 		}
 	}
-	g_config = config_new();
+	if (optind == argc) {
+		fprintf(stderr, "please specify a package to reconfigure\n");
+		exit(1);
+	}
+	
+	/* Default is to force showing of old questions by default
+	 * for reconfiguring, and show low priority questions. */
+	if (! unseen_only)
+		g_config->set(g_config, "_cmdline::showold", "true");
+	if (priority_override)
+		g_config->set(g_config, "_cmdline::priority", priority_override);
+	else if (! default_priority && (getenv("DEBIAN_PRIORITY") == NULL))
+		g_config->set(g_config, "_cmdline::priority", "low");
 
 	/* parse the configuration info */
 	if (g_config->read(g_config, DEBCONFCONFIG) == 0)
@@ -396,7 +402,7 @@ int main(int argc, char **argv)
 	/* initialize database and frontend modules */
 	if ((g_templates = template_db_new(g_config, NULL)) == 0)
 		DIE("Cannot initialize DebConf templates database");
-    if ((g_questions = question_db_new(g_config, g_templates, NULL)) == 0)
+	if ((g_questions = question_db_new(g_config, g_templates, NULL)) == 0)
 		DIE("Cannot initialize DebConf config database");
 	if ((g_frontend = frontend_new(g_config, g_templates, g_questions)) == 0)
 		DIE("Cannot initialize DebConf frontend");
