@@ -7,7 +7,7 @@
  *
  * Description: Newt UI for cdebconf
  *
- * $Id: newt.c 59368 2009-07-16 15:18:17Z cjwatson $
+ * $Id: newt.c 61397 2009-11-19 22:10:05Z fjp $
  *
  * cdebconf is (c) 2000-2001 Randolph Chung and others under the following
  * license.
@@ -71,7 +71,7 @@ struct newt_data {
                   scale_cancel,
                   perc_label;
     int           scale_textbox_height;
-    char *        scale_info;
+    struct question *scale_info;
 };
 
 struct newtColors newtAltColorPalette = {
@@ -197,13 +197,14 @@ cdebconf_newt_setup(void)
 /* cdebconf-newt-terminal needs this in order to be able to restore the
  * display properly after tearing down the terminal.
  */
-char *
+struct question *
 cdebconf_newt_get_progress_info(struct frontend *obj)
 {
     struct newt_data *data = (struct newt_data *)obj->data;
-    if (data->scale_info)
-        return strdup(data->scale_info);
-    else
+    if (data->scale_info) {
+        question_ref(data->scale_info);
+        return data->scale_info;
+    } else
         return NULL;
 }
 
@@ -376,6 +377,7 @@ show_separate_window(struct frontend *obj, struct question *q, int is_help)
     // buttons
     int extra = 3;
     int format_note = 0;
+    int t_voffset = 0;
     char *full_description;
     int ret;
 #ifdef HAVE_LIBTEXTWRAP
@@ -394,7 +396,8 @@ show_separate_window(struct frontend *obj, struct question *q, int is_help)
     if (strcmp(q->template->type, "note") == 0 || strcmp(q->template->type, "error") == 0)
     {
         format_note = 1;
-        extra++;
+        t_voffset = 1;
+        extra += 2;
         full_description = strdup(ext_descr);
     }
     else
@@ -438,8 +441,8 @@ show_separate_window(struct frontend *obj, struct question *q, int is_help)
     cdebconf_newt_create_window(win_width, win_height, obj->title, q->priority);
     form = cdebconf_newt_create_form(NULL);
     if (format_note)
-        newtFormAddComponent(form, newtLabel((win_width - strwidth(descr))/2, 0, descr));
-    textbox = newtTextbox(TEXT_PADDING, 1, t_width, t_height, flags);
+        newtFormAddComponent(form, newtLabel((win_width - strwidth(descr))/2, 1, descr));
+    textbox = newtTextbox(TEXT_PADDING, 1 + t_voffset, t_width, t_height, flags);
     assert(textbox);
     newtTextboxSetText(textbox, full_description);
     free(full_description);
@@ -1155,9 +1158,6 @@ newt_initialize(struct frontend *obj, struct configuration *conf)
         newtAltColorPalette = newtDefaultColorPalette;
     newtSetColors(newtAltColorPalette);
     newtGetScreenSize(&width, &height);
-    // Fill the screen so people can shift-pgup properly
-    for (i = 0; i < height; i++)
-        putchar('\n');
     newtFinished();
     return DC_OK;
 }
@@ -1283,12 +1283,13 @@ newt_lookup_directive(struct frontend *obj, const char *directive)
 }
 
 static void
-newt_make_progress_bar(struct frontend *obj, const char *info)
+newt_make_progress_bar(struct frontend *obj, struct question *info)
 {
     struct newt_data *data = (struct newt_data *)obj->data;
     int width = 80, height = 24, win_width, win_height, text_height;
     int extra = 0;
     bool can_cancel_progress;
+    char *info_desc = NULL;
     bool changed = false;
 #ifdef HAVE_LIBTEXTWRAP
     textwrap_t tw;
@@ -1305,10 +1306,14 @@ newt_make_progress_bar(struct frontend *obj, const char *info)
     win_width = width-7;
     if (!info)
         info = data->scale_info;
-    else
-        data->scale_info = strdup(info);
+    else {
+        data->scale_info = info;
+        question_ref(data->scale_info);
+    }
     if (info)
-        text_height = cdebconf_newt_get_text_height(info, win_width);
+        info_desc = q_get_raw_description(info);
+    if (info_desc)
+        text_height = cdebconf_newt_get_text_height(info_desc, win_width);
     else
         text_height = 0;
     /* Minimal height set to 2 to prevent box flashing */
@@ -1327,11 +1332,14 @@ newt_make_progress_bar(struct frontend *obj, const char *info)
         }
     }
     if (!data->scale_form || changed) {
+        char *title_desc;
         if (text_height + 3 + extra <= height - 5)
             win_height = text_height + 3 + extra;
         else
             win_height = height - 5;
-        newtCenteredWindow(win_width, win_height, obj->progress_title);
+        title_desc = q_get_raw_description(obj->progress_title);
+        newtCenteredWindow(win_width, win_height, title_desc);
+        free(title_desc);
         data->scale_bar = newtScale(TEXT_PADDING, 1, win_width-2*TEXT_PADDING, obj->progress_max - obj->progress_min);
         data->scale_textbox = newtTextbox(TEXT_PADDING, 3, win_width-2*TEXT_PADDING, text_height, flags);
         data->scale_textbox_height = text_height;
@@ -1345,21 +1353,23 @@ newt_make_progress_bar(struct frontend *obj, const char *info)
         newtFormSetTimer(data->scale_form, 1);
     }
     newtScaleSet(data->scale_bar, obj->progress_cur - obj->progress_min);
-    if (info) {
+    if (info_desc) {
 #ifdef HAVE_LIBTEXTWRAP
         textwrap_init(&tw);
         textwrap_columns(&tw, win_width - 2 - 2*TEXT_PADDING);
-        wrappedtext = textwrap(&tw, info);
+        wrappedtext = textwrap(&tw, info_desc);
         newtTextboxSetText(data->scale_textbox, wrappedtext);
         free(wrappedtext);
 #else
-        newtTextboxSetText(data->scale_textbox, info);
+        newtTextboxSetText(data->scale_textbox, info_desc);
 #endif
     }
+    free(info_desc);
 }
 
 static void
-newt_progress_start(struct frontend *obj, int min, int max, const char *title)
+newt_progress_start(struct frontend *obj, int min, int max,
+                    struct question *title)
 {
     struct newt_data *data = (struct newt_data *)obj->data;
 
@@ -1368,8 +1378,9 @@ newt_progress_start(struct frontend *obj, int min, int max, const char *title)
          * use one anyway, tear down the old progress bar first.
          */
         newt_progress_stop(obj);
-    DELETE(obj->progress_title);
-    obj->progress_title = strdup(title);
+    question_deref(obj->progress_title);
+    obj->progress_title = title;
+    question_ref(obj->progress_title);
     obj->progress_min = min;
     obj->progress_max = max;
     obj->progress_cur = min;
@@ -1423,7 +1434,7 @@ newt_progress_set(struct frontend *obj, int val)
 }
 
 static int
-newt_progress_info(struct frontend *obj, const char *info)
+newt_progress_info(struct frontend *obj, struct question *info)
 {
     struct newt_data *data = (struct newt_data *)obj->data;
     struct newtExitStruct es;
@@ -1456,7 +1467,7 @@ newt_progress_stop(struct frontend *obj)
         newtPopWindow();
         newtFinished();
         data->scale_form = data->scale_bar = data->perc_label = data->scale_textbox = data->scale_cancel = NULL;
-        free(data->scale_info);
+        question_deref(data->scale_info);
         data->scale_info = NULL;
     }
 }
