@@ -7,7 +7,7 @@
  *
  * Description: SLang-based cdebconf UI module
  *
- * $Id: slang.c 11913 2004-03-25 04:52:51Z klfmanik-guest $
+ * $Id: slang.c 59807 2009-07-28 16:06:55Z cjwatson $
  *
  * cdebconf is (c) 2000-2001 Randolph Chung and others under the following
  * license.
@@ -58,12 +58,6 @@
 #define COLS		(SLtt_Screen_Cols ? SLtt_Screen_Cols : 80)
 #define UIDATA(obj) 	((struct uidata *)(obj)->data)
 
-#define q_get_extended_description(q)   question_get_field((q), "", "extended_description")
-#define q_get_description(q)  		question_get_field((q), "", "description")
-#define q_get_choices(q)		question_get_field((q), "", "choices")
-#define q_get_choices_vals(q)		question_get_field((q), NULL, "choices")
-#define q_get_indices(q)		question_get_field((q), "", "indices")
-
 /* Private variables */
 struct uidata {
 	struct slwindow qrywin, descwin, progwin;
@@ -208,8 +202,8 @@ static void slang_wrapprint(struct slwindow *win, const char *str, int start)
 static void slang_drawdesc(struct frontend *ui, struct question *q)
 {
 	struct uidata *uid = UIDATA(ui);
-	char *descr = q_get_description(q);
-	char *ext_descr = q_get_extended_description(q);
+	char *descr = q_get_description(ui, q);
+	char *ext_descr = q_get_extended_description(ui, q);
 
 	/* Clear the windows */
 	slang_drawwin(&uid->qrywin);
@@ -223,6 +217,11 @@ static void slang_drawdesc(struct frontend *ui, struct question *q)
 	/* caller should call slang_flush() ! */
 }
 
+#ifdef __GNUC__
+static void slang_printf(int r, int c, int col, char *fmt, ...)
+	__attribute__((format(printf, 4, 5)));
+#endif
+
 /*
  * slang_printf
  *
@@ -291,14 +290,14 @@ static int slang_keyhandler(struct frontend *ui, struct question *q, int *pos,
 static char *get_text(struct frontend *obj, const char *template, char *fallback)
 {
         struct question *q = obj->qdb->methods.get(obj->qdb, template);
-        return q ? q_get_description(q) : fallback;
+        return q ? q_get_description(obj, q) : fallback;
 }
 
 static char *button_text(struct frontend *obj, const char *template, char *fallback)
 {
 	char text[50];
         struct question *q = obj->qdb->methods.get(obj->qdb, template);
-	sprintf(text, "< %s >", q ? q_get_description(q) : fallback);
+	sprintf(text, "< %s >", q ? q_get_description(obj, q) : fallback);
 	return strdup(text);
 }
 
@@ -369,7 +368,7 @@ static int slang_boolean(struct frontend *ui, struct question *q)
 	if (!yes_text)	yes_text = get_text(ui, "debconf/button-yes", "Yes");
 	if (!no_text)  	no_text = get_text(ui, "debconf/button-no", "No");
 
-	value = question_get_field(q, NULL, "value");
+	value = question_get_field(ui, q, "C", "value");
 
 	ans = (strcmp(value, "true") == 0);
 	pos = (ans ? 2 : 3);
@@ -423,27 +422,32 @@ static int slang_note(struct frontend *ui, struct question *q)
 
 static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 {
-	char *choices[100] = {0};
-	char *choices_translated[100] = {0};
-	char *defaults[100] = {0};
-	char selected[100] = {0};
+	char **choices;
+	char **choices_translated;
+	char **defaults;
+	char *selected;
 	char answer[1024] = {0};
 	int *tindex = NULL;
-	const char *indices = q_get_indices(q);
+	const char *indices = q_get_indices(ui, q);
 	int i, j, count, dcount, ret = 0, val = 0, pos = 2, xpos, ypos;
 	int top, bottom, longest, ch;
 	struct uidata *uid = UIDATA(ui);
 	struct slwindow *win = &uid->qrywin;
 
 	/* Parse out all the choices */
-	count = strgetargc(q_get_choices_vals(q));
+	count = strgetargc(q_get_choices_vals(ui, q));
 	if (count <= 0)
 		return DC_NOTOK;
+	choices = malloc(sizeof(char *) * count);
+	choices_translated = malloc(sizeof(char *) * count);
 	tindex = malloc(sizeof(int) * count);
-	strchoicesplitsort(q_get_choices_vals(q), q_get_choices(q), indices, choices, choices_translated, tindex, DIM(choices_translated));
-	dcount = strchoicesplit(question_get_field(q, NULL, "value"), defaults, DIM(defaults));
-	INFO(INFO_VERBOSE, "Parsed out %d choices, %d defaults\n", count, dcount);
-	if (count <= 0) return DC_NOTOK;
+	if (strchoicesplitsort(q_get_choices_vals(ui, q), q_get_choices(ui, q), indices, choices, choices_translated, tindex, count) != count)
+		return DC_NOTOK;
+
+	defaults = malloc(sizeof(char *) * count);
+	dcount = strchoicesplit(question_get_field(ui, q, "C", "value"), defaults, count);
+	INFO(INFO_VERBOSE, "Parsed out %d choices, %d defaults", count, dcount);
+	if (dcount <= 0) return DC_NOTOK;
 	if (count == 1 && !multi)
 	{
 		dcount = 1;
@@ -455,16 +459,20 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 	 * Loop in descending order so that cursor is at the first
 	 * selected value
 	 */
+	selected = malloc(sizeof(char) * count);
 	for (i = count-1; i >=0; i--)
+	{
+		selected[tindex[i]] = 0;
 		for (j = 0; j < dcount; j++)
 			if (strcmp(choices[tindex[i]], defaults[j]) == 0)
 			{
 				selected[tindex[i]] = 1;
 				val = i;
 			}
+	}
 
 	longest = strlongest(choices, count);
-	top = 0;
+	top = val - win->h + 8;
 	if (top < 0) top = 0;
 	xpos = (COLS-longest)/2-1;
 
@@ -473,7 +481,7 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 		ypos = 3;
 		bottom = top + MIN(count, win->h - 7);
 
-		INFO(INFO_VERBOSE, "[val; %d, top: %d, bottom: %d]\n", val,
+		INFO(INFO_VERBOSE, "[val: %d, top: %d, bottom: %d]", val,
 			top, bottom);
 		for (i = top; i < bottom; i++)
 		{
@@ -482,7 +490,7 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 				"(%c) %-*s ", (selected[tindex[i]] ? '*' : ' '), 
 				longest, choices_translated[i]);
 
-			INFO(INFO_VERBOSE, "(%c) %-*s\n", (selected[tindex[i]] ? '*' : 
+			INFO(INFO_VERBOSE, "(%c) %-*s", (selected[tindex[i]] ? '*' : 
 				' '), longest, choices_translated[i]);
 		}
 
@@ -528,7 +536,7 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 			default: 
 				if (multi == 0)
 				{
-					memset(selected, 0, sizeof(selected));
+					memset(selected, 0, sizeof(*selected) * count);
 					selected[tindex[val]] = 1;
 				}
 				else
@@ -551,8 +559,12 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 		free(choices[i]);
 		free(choices_translated[i]);
 	}
+	free(choices);
+	free(choices_translated);
 	for (i = 0; i < dcount; i++)
 		free(defaults[i]);
+	free(defaults);
+	free(selected);
 	question_setvalue(q, answer);
 
 	free(tindex);
@@ -562,7 +574,7 @@ static int slang_getselect(struct frontend *ui, struct question *q, int multi)
 
 static int slang_select(struct frontend *ui, struct question *q)
 {
-	INFO(INFO_VERBOSE, "calling getselect\n");
+	INFO(INFO_VERBOSE, "calling getselect");
 	return slang_getselect(ui, q, 0);
 }
 
@@ -580,7 +592,7 @@ static int slang_getstring(struct frontend *ui, struct question *q, char showch)
 	int xpos = 0, ypos = win->y + win->border + 4;
 	int cursor;
 
-	STRCPY(value, question_get_field(q, NULL, "value"));
+	STRCPY(value, question_get_field(ui, q, "C", "value"));
 	cursor = strlen(value);
 
 	/* TODO: scrolling */
@@ -722,12 +734,15 @@ static int slang_initialize(struct frontend *obj, struct configuration *cfg)
 	obj->data = uid;
 
 	SLtt_get_terminfo();
+#if SLANG_VERSION > 20000
+	SLutf8_enable(-1);
+#endif
 	ret = SLsmg_init_smg();
-	INFO(INFO_DEBUG, "SLsmg_init_smg returned %d\n", ret);
+	INFO(INFO_DEBUG, "SLsmg_init_smg returned %d", ret);
 	ret = SLang_init_tty(-1, 0, 0);
-	INFO(INFO_DEBUG, "SLang_init_tty returned %d\n", ret);
+	INFO(INFO_DEBUG, "SLang_init_tty returned %d", ret);
 	ret = SLkp_init();
-	INFO(INFO_DEBUG, "SLkp_init returned %d\n", ret);
+	INFO(INFO_DEBUG, "SLkp_init returned %d", ret);
 
 	slang_initwin(obj, WIN_QUERY, &uid->qrywin);
 	slang_initwin(obj, WIN_DESC, &uid->descwin);
@@ -751,7 +766,7 @@ static int slang_shutdown(struct frontend *obj)
 	return DC_OK;
 }
 
-static int slang_can_go_back(struct frontend *obj, struct question *q)
+static bool slang_can_go_back(struct frontend *obj, struct question *q)
 {
 	return (obj->capability & DCF_CAPB_BACKUP);
 }
@@ -769,14 +784,14 @@ static int slang_go(struct frontend *obj)
 		ret = DC_OK;
 		for (i = 0; i < DIM(question_handlers); i++)
 		{
-			INFO(INFO_VERBOSE, "For question [%s], comparing question type [%s] against handler %d (%s)\n", q->tag, q->template->type, i, question_handlers[i].type);
+			INFO(INFO_VERBOSE, "For question [%s], comparing question type [%s] against handler %d (%s)", q->tag, q->template->type, i, question_handlers[i].type);
 			if (strcasecmp(q->template->type, question_handlers[i].type) == 0)
 			{
-				INFO(INFO_VERBOSE, "Found one!\n");
+				INFO(INFO_VERBOSE, "Found one!");
 				UIDATA(obj)->descstart = 0;
 				slang_drawdesc(obj, q);
 				ret = question_handlers[i].handler(obj, q);
-				INFO(INFO_VERBOSE, "Return code = %d\n", ret);
+				INFO(INFO_VERBOSE, "Return code = %d", ret);
 				switch (ret)
 				{
 				case DC_OK:
@@ -797,6 +812,11 @@ static int slang_go(struct frontend *obj)
 				break;
 			}
 		}
+		if (i == DIM(question_handlers)) {
+		    	SLsmg_reset_smg();
+			SLang_reset_tty();
+			return DC_NOTIMPL;
+		}
 		if (ret == DC_OK)
 			q = q->next;
 	}
@@ -806,26 +826,30 @@ static int slang_go(struct frontend *obj)
 	return DC_OK;
 }
 
-static void
+static int
 slang_progress_set(struct frontend *obj, int val);
 
 static void
-slang_progress_start(struct frontend *obj, int min, int max, const char *title)
+slang_progress_start(struct frontend *obj, int min, int max, struct question *title)
 {
 	struct uidata *uid = UIDATA(obj);
+	char *title_desc;
 	
-	DELETE(obj->progress_title);
-	obj->progress_title = strdup(title);
+	question_deref(obj->progress_title);
+	obj->progress_title = title;
+	question_ref(obj->progress_title);
 	obj->progress_min = min;
 	obj->progress_max = max;
 	
 	slang_drawwin(&uid->descwin);
-	slang_wrapprint(&uid->descwin, title, 0);
+	title_desc = q_get_raw_description(title);
+	slang_wrapprint(&uid->descwin, title_desc, 0);
+	free(title_desc);
 
 	slang_progress_set(obj, min);
 }
 
-static void
+static int
 slang_progress_set(struct frontend *obj, int val)
 {
 	struct uidata *uid = UIDATA(obj);
@@ -868,24 +892,32 @@ slang_progress_set(struct frontend *obj, int val)
 	}
 	
 	free(percstr);
+
+	return DC_OK;
 }
 
-static void
-slang_progress_info(struct frontend *obj, const char *info)
+static int
+slang_progress_info(struct frontend *obj, struct question *info)
 {
 	struct uidata *uid = UIDATA(obj);
+	char *title_desc = q_get_raw_description(obj->progress_title);
+	char *info_desc = q_get_raw_description(info);
 	char *buf = (char *)
-		malloc(strlen(obj->progress_title) + strlen(info) + 3);
+		malloc(strlen(title_desc) + strlen(info_desc) + 3);
 	
-	strcpy(buf, obj->progress_title);
+	strcpy(buf, title_desc);
 	strcat(buf, ":\n");
-	strcat(buf, info);
+	strcat(buf, info_desc);
+	free(info_desc);
+	free(title_desc);
 	
 	slang_drawwin(&uid->descwin);
 	slang_wrapprint(&uid->descwin, buf, 0);
 	slang_flush();
 
 	free(buf);
+
+	return DC_OK;
 }
 
 static void
